@@ -602,14 +602,6 @@ def zoom(input, zoom, output=None, order=3, mode='constant', cval=0.0,
     output_shape = tuple(
             [int(round(ii * jj)) for ii, jj in zip(input.shape, zoom)])
 
-    output_shape_old = tuple(
-            [int(ii * jj) for ii, jj in zip(input.shape, zoom)])
-    if output_shape != output_shape_old:
-        warnings.warn(
-                "From scipy 0.13.0, the output shape of zoom() is calculated "
-                "with round() instead of int() - for these inputs the size of "
-                "the returned array has changed.", UserWarning)
-
     zoom_div = numpy.array(output_shape, float) - 1
     # Zooming to infinite values is unpredictable, so just choose
     # zoom factor 1 instead
