@@ -34,7 +34,7 @@ from .optimize import OptimizeResult, OptimizeWarning, _check_unknown_options
 from ._linprog_util import _postsolve
 
 
-def _pivot_col(T, tol=1.0E-12, bland=False):
+def _pivot_col(T, tol=1e-9, bland=False):
     """
     Given a linear programming simplex tableau, determine the column
     of the variable to enter the basis.
@@ -95,7 +95,7 @@ def _pivot_col(T, tol=1.0E-12, bland=False):
     return True, np.ma.nonzero(ma == ma.min())[0][0]
 
 
-def _pivot_row(T, basis, pivcol, phase, tol=1.0E-12, bland=False):
+def _pivot_row(T, basis, pivcol, phase, tol=1e-9, bland=False):
     """
     Given a linear programming simplex tableau, determine the row for the
     pivot operation.
@@ -166,7 +166,7 @@ def _pivot_row(T, basis, pivcol, phase, tol=1.0E-12, bland=False):
     return True, min_rows[0]
 
 
-def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
+def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-9):
     """
     Pivot the simplex tableau inplace on the element given by (pivrow, pivol).
     The entering variable corresponds to the column given by pivcol forcing
@@ -226,11 +226,12 @@ def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
             "Removing redundant constraints, changing the pivot strategy "
             "via Bland's rule or increasing the tolerance may "
             "help reduce the issue.".format(pivval, tol))
-        warn(message, OptimizeWarning)
+        warn(message, OptimizeWarning, stacklevel=5)
 
 
-def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
-                   callback=None, tol=1.0E-12, nit0=0, bland=False, _T_o=None):
+def _solve_simplex(T, n, basis, callback, postsolve_args,
+                   maxiter=1000, tol=1e-9, phase=2, bland=False, nit0=0,
+                   ):
     """
     Solve a linear programming problem in "standard form" using the Simplex
     Method. Linear Programming is intended to solve the following problem form:
@@ -278,13 +279,6 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
         An array of the indices of the basic variables, such that basis[i]
         contains the column corresponding to the basic variable for row i.
         Basis is modified in place by _solve_simplex
-    maxiter : int
-        The maximum number of iterations to perform before aborting the
-        optimization.
-    phase : int
-        The phase of the optimization being executed. In phase 1 a basic
-        feasible solution is sought and the T has an additional row
-        representing an alternate objective function.
     callback : callable, optional
         If a callback function is provided, it will be called within each
         iteration of the algorithm. The callback must accept a
@@ -321,17 +315,27 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                 The number of iterations performed.
             message : str
                 A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
+    maxiter : int
+        The maximum number of iterations to perform before aborting the
+        optimization.
     tol : float
         The tolerance which determines when a solution is "close enough" to
         zero in Phase 1 to be considered a basic feasible solution or close
         enough to positive to serve as an optimal solution.
-    nit0 : int
-        The initial iteration number used to keep an accurate iteration total
-        in a two-phase problem.
+    phase : int
+        The phase of the optimization being executed. In phase 1 a basic
+        feasible solution is sought and the T has an additional row
+        representing an alternate objective function.
     bland : bool
         If True, choose pivots using Bland's rule [3]_. In problems which
         fail to converge due to cycling, using Bland's rule can provide
         convergence at the expense of a less optimal path about the simplex.
+    nit0 : int
+        The initial iteration number used to keep an accurate iteration total
+        in a two-phase problem.
 
     Returns
     -------
@@ -349,6 +353,8 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
 
     """
     nit = nit0
+    status = 0
+    message = ''
     complete = False
 
     if phase == 1:
@@ -373,7 +379,7 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                             if abs(T[pivrow, col]) > tol]
             if len(non_zero_row) > 0:
                 pivcol = non_zero_row[0]
-                _apply_pivot(T, basis, pivrow, pivcol)
+                _apply_pivot(T, basis, pivrow, pivcol, tol)
                 nit += 1
 
     if len(basis[:m]) == 0:
@@ -401,9 +407,8 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
             solution[:] = 0
             solution[basis[:n]] = T[:n, -1]
             x = solution[:m]
-            c, A_ub, b_ub, A_eq, b_eq, bounds, undo = _T_o
             x, fun, slack, con, _, _ = _postsolve(
-                x, c, A_ub, b_ub, A_eq, b_eq, bounds, undo=undo, tol=tol
+                x, postsolve_args, tol=tol
             )
             res = OptimizeResult({
                 'x': x,
@@ -425,13 +430,14 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                 status = 1
                 complete = True
             else:
-                _apply_pivot(T, basis, pivrow, pivcol)
+                _apply_pivot(T, basis, pivrow, pivcol, tol)
                 nit += 1
     return nit, status
 
 
-def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
-                     tol=1.0E-12, bland=False, _T_o=None, **unknown_options):
+def _linprog_simplex(c, c0, A, b, callback, postsolve_args,
+                     maxiter=1000, tol=1e-9, disp=False, bland=False,
+                     **unknown_options):
     """
     Minimize a linear objective function subject to linear equality and
     non-negativity constraints using the two phase simplex method.
@@ -491,6 +497,9 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
                 The number of iterations performed.
             message : str
                 A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
 
     Options
     -------
@@ -507,6 +516,10 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
         prevent cycling. If False, choose pivots which should lead to a
         converged solution more quickly. The latter method is subject to
         cycling (non-convergence) in rare instances.
+    unknown_options : dict
+        Optional arguments not used by this particular solver. If
+        `unknown_options` is non-empty a warning is issued listing all
+        unused options.
 
     Returns
     -------
@@ -605,8 +618,11 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
     row_pseudo_objective[av] = 0
     T = np.vstack((row_constraints, row_objective, row_pseudo_objective))
 
-    nit1, status = _solve_simplex(T, n, basis, phase=1, callback=callback,
-                                  maxiter=maxiter, tol=tol, bland=bland, _T_o=_T_o)
+    nit1, status = _solve_simplex(T, n, basis, callback=callback,
+                                  postsolve_args=postsolve_args,
+                                  maxiter=maxiter, tol=tol, phase=1,
+                                  bland=bland
+                                  )
     # if pseudo objective is zero, remove the last row from the tableau and
     # proceed to phase 2
     nit2 = nit1
@@ -630,9 +646,11 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
 
     if status == 0:
         # Phase 2
-        nit2, status = _solve_simplex(T, n, basis, maxiter=maxiter,
-                                      phase=2, callback=callback, tol=tol,
-                                      nit0=nit1, bland=bland, _T_o=_T_o)
+        nit2, status = _solve_simplex(T, n, basis, callback=callback,
+                                      postsolve_args=postsolve_args,
+                                      maxiter=maxiter, tol=tol, phase=2,
+                                      bland=bland, nit0=nit1
+                                      )
 
     solution = np.zeros(n + m)
     solution[basis[:n]] = T[:n, -1]
